--
-- coup
--
-- Copyright (c) 2014-2016, rnlf
--
-- This library is free software; you can redistribute it and/or modify it
-- under the terms of the MIT license. See LICENSE for details.
--

local BASE = (...):match("(.-)[^%.]+$")
local tables = require(BASE .. ".tables")
local cache = require(BASE .. ".cache")

local Tilelayer = {}
Tilelayer.__index = Tilelayer

function Tilelayer:foreachInRange(x1, y1, x2, y2, func)
  local m = self.map
  local w = m.width
  for y = y1, y2 do
    for x = x1, x2 do
      local t = self.data[y*w+x+1]

      if t and t ~= 0 then
        local img, q = m:get_tile(t)
        func(img, x, y, q)
      end
    end
  end
end


function Tilelayer:on_update(func)
  self.update_func = func
end



local Tilemap = {
  width = function(self)
    return self.data.width
  end,

  height = function(self)
    return self.data.height
  end,

  dimensions = function(self)
    return self.data.width, self.data.height
  end,

  get_layer = function(self, name)
    return self.layers[name]
  end,

  add_tileset = function(self, tileset)
    local si = tileset.first
    local tc = tileset.tileCount
    for i=si,si+tc do
      self.tilesets[i] = tileset
    end
  end,

  add_layer = function(self, name, data)
    self.layers[name] = setmetatable({
      data = data,
      map = self,
    }, Tilelayer)
  end,

  get_tile = function(self, idx)
    local ts = self.tilesets[idx]
    return ts.image, ts:get(idx)
  end
}
Tilemap.__index = Tilemap

return {Tilemap = function(tilewidth, tileheight, width, height)
  local new = {
    tilewidth = tilewidth,
    tileheight = tileheight,
    width = width,
    height = height,
    tilesets = {},
    layers = {}
  }
  return setmetatable(new, Tilemap)
end}

-- TODO Write tests :-)
