local Enemy = {}
Enemy.__index = Enemy

local enemyUpdateRate = 20
local maxFear = 5000


local attackMessage = function(enemy, state)
    local msgs = {
      "Don't be alarmed. We are just testing our new weapon systems.",
      "We mean you no harm. We are just doing the annual inspection of our weapons of mass destruction.",
      "You have become weak. You are no threat to us anymore.",
    }
    if enemy.lastMessage < enemy.lastUpdate - 5 then
      state.teletype:print(string.format("Message from hostile president: '%s'", msgs[math.random(1, #msgs)]))
      enemy.lastMessage = enemy.lastUpdate
    end
  end


local warningMessage = function(enemy, state)
    local msgs = {
      "Your hostile actions have not been unnoticed!",
      "Be assured that our missiles are fully armed!",
      "We will not hesitate to destroy you!",
      "We are not going down alone!",
      "We have maximum secondary strike capabilities!",
      "Power down your ICBMs immediately or face the consequences!",
      "I hope you have said your prayers!",
      "If this is what you want, then so be it!",
    }
    if enemy.lastMessage < enemy.lastUpdate - 5 then
      state.teletype:print(string.format("Message from hostile president: '%s'", msgs[math.random(1, #msgs)]))
      enemy.lastMessage = enemy.lastUpdate
    end
  end

local relieveMessage = function(enemy, state)
    local msgs = {
      "I see that you are reasonable.",
      "That was close. Let us not get into such a dilemma again, friend.",
      "We will power down our missiles as a sign of friendship."
    }
    state.teletype:print(string.format("Message from hostile president: '%s'", msgs[math.random(1, #msgs)]))
 end

 local jokingMessage = function(enemy, state)
    local msgs = {
      "It was just a demonstration of power. We will shut down some of our missiles now.",
      "That was only a manoeuver. Don't wet yourself, my friend. It's over, alright?",
      "We just wanted to show you what we are capable of. Do not mess with us.",
      "Were you afraid? We are already shutting down our missiles again."
    }
    state.teletype:print(string.format("Message from hostile president: '%s'", msgs[math.random(1, #msgs)]))
  end

function Enemy:estimateState(state)
  local t = math.floor(state.clock.time * enemyUpdateRate) 
  if self.lastUpdate ~= t then
    self.lastUpdate = t

    self.fearFactor = self.fearFactor + 1

    local factor = 0.5
    local threat = self:estimateThreat(state)

    if threat == 10000 then
      state.teletype:clearBuffer()
      state.teletype:print(" ")
      state.teletype:print(" ")
      state.teletype:print("Message from hostile president: 'We knew you would do this one day. TO DEATH!'")
      state.teletype:print(" ")
      state:enemyLaunchesNuke()
      return
    end

    local threatFactor = factor * self.fearFactor + 1
    threat = threat * threatFactor

    self.fear = self.fear + threat

    if threat < 10 then
      self.fear = self.fear * 0.92
    end

    self.fear = math.min(math.max(0, self.fear), maxFear)
    for i = #self.assets, 1, -1 do
      local a = self.assets[i]
      if a.action.check(a.state, self, state) then
        table.remove(self.assets, i)
      end
    end
    
    if math.random() < 0.6 then
      if self.fear < 1500 then
        self:startNotAfraidAction(state)
      elseif self.fear > maxFear - 1500 then
        self:startAfraidAction(state)
      end
    end

    if self.fear > maxFear - 2000 then
      warningMessage(self, state)
    elseif self.fear < 3500 and self.lastFear >= 3500 then
      relieveMessage(self, state)
    elseif self.fear > 1500 and self.lastFear <= 1500 then
      jokingMessage(self, state)
    elseif self.fear < 2000 then
      attackMessage(self, state)
    end

    self.lastFear = self.fear

    if self.fear == 5000 then
      state.teletype:clearBuffer()
      state.teletype:print(" ")
      state.teletype:print(" ")
      state.teletype:print("Message from hostile president: 'Face your destiny! Now you get what you deserve!'")
      state.teletype:print(" ")
      state:enemyLaunchesNuke()
      self.reason = "fear"
    elseif self.fear > 4500 and state.doors == 10 and state.power == 1 and state.fuelPumps == 3 and state.buttonState == 19 then
      state.teletype:clearBuffer()
      state.teletype:print(" ")
      state.teletype:print(" ")
      state.teletype:print("Message from hostile president: 'At least I will not die alone! FIRE MISSILES!'")
      state.teletype:print(" ")
      state:enemyLaunchesNuke()
      self.reason = "fear"
    end

    if self.fear == 0 then
      state.teletype:clearBuffer()
      state.teletype:print(" ")
      state.teletype:print(" ")
      state.teletype:print("Message from hostile president: 'We will end this unfortunate situation right now.'")
      state.teletype:print(" ")
      state:enemyLaunchesNuke()
      self.reason = "weakness"
    elseif self.fear < 500 and (state.doors ~= 10 or state.power == 0 or state.fuelPumps ~= 3) then
      state.teletype:clearBuffer()
      state.teletype:print(" ")
      state.teletype:print(" ")
      state.teletype:print("Message from hostile president: 'You are weak! We will rid the world of your presence now!'")
      state.teletype:print(" ")
      state:enemyLaunchesNuke()
      self.reason = "weakness"
    end



  end
end

local surfaceSub = 
{
    start = function(enemy, state)
      local N = math.random(0, 90)
      local E = math.random(0, 180)
      local ns = math.random(1,2)
      local ew = math.random(1,2)
      local nsd = {"N", "S"}
      local ewd = {"E", "W"}
      local ss = math.random(1, 9)
      local location = string.format("%s%02d, %s%03d", nsd[ns], N, ewd[ew], E)
      state.teletype:print("SPYSAT-" .. ss .. ": Hostile submarine surfaced (" .. location .. ")")
      return {ss, location, 1}
    end,

    check = function(ownState, enemy, state)
      if enemy.fear > 1500 and math.random() > enemy.fear / 1500.0  then
        state.teletype:print("SPYSAT-" .. ownState[1] .. ": Submarine submerged (" .. ownState[2] .. ")")
        return true
      end

      if enemy.fear > 1500 and enemy.fear < 3500 and math.random() < 0.1 then
        local msgs = {
          "SPYSAT-%d: Submarine submerged (%s)",
          "SPYSAT-%d: Submarine closed missile bays (%s)"
        }
        state.teletype:print(string.format(msgs[ownState[3]], ownState[1], ownState[2]))
        ownState[3] = ownState[3]- 1
        return ownState[3] == 0
      elseif ownState[3] < 2 and 
          ((enemy.fear < 1000 and math.random() > enemy.fear / 500.0) or
           (enemy.fear > 4000 and math.random() < (enemy.fear - 4000) / 500.0)) then

        local msgs = {
          "SPYSAT-%d: Submarine opened missile bays (%s)"
        }
        state.teletype:print(string.format("SPYSAT-%d: Submarine opened missile bays (%s)", ownState[1], ownState[2]))
        ownState[3] = ownState[3] + 1
      end
    end
  }

local powerupSilo =
  {
    start = function(enemy, state)
      local r = {math.random(1, 26), math.random(1,26), math.random(1, 2000)}
      local s = string.byte("A")-1
      local name = string.format("%s%s-%04d", string.char(s+r[1]), string.char(s+r[2]), r[3])
      state.teletype:print("Missile silo " .. name .. " powered up")
      return {name = name, state = 1}
    end,

    check = function(ownState, enemy, state)
      if enemy.fear > 1500 and enemy.fear < 3500 and math.random() < 0.1 then
        local msgs = {
          "Missile silo %s powered down",
          "Missile silo %s shut down fuel pumps",
          "Missile silo %s closed blast doors"
        }
        state.teletype:print(string.format(msgs[ownState.state], ownState.name))
        ownState.state = ownState.state - 1
        return ownState.state == 0
      elseif ownState.state < 3 and 
          ((enemy.fear < 1000 and math.random() > enemy.fear / 500.0) or
           (enemy.fear > 4000 and math.random() < (enemy.fear - 4000) / 500.0)) then

        local msgs = {
          "Missile silo %s started missile fuel pumps",
          "Missile silo %s opened blast doors"
        }
        state.teletype:print(string.format(msgs[ownState.state], ownState.name))
        ownState.state = ownState.state + 1
      end
    end
  }





local notAfraidActions = {
  surfaceSub,
  powerupSilo
}

local afraidActions = {
  surfaceSub,
  powerupSilo,
}


function Enemy:startNotAfraidAction(state)
  local n = #notAfraidActions
  local a = math.random(1, n)

  local action = notAfraidActions[a]
  local actionState = action.start(self, state)
  self.assets[#self.assets+1] = {action=action, state=actionState}
end

function Enemy:startAfraidAction(state)
  local n = #afraidActions
  local a = math.random(1, n)

  local action = afraidActions[a]
  local actionState = action.start(self, state)
  self.assets[#self.assets+1] = {action=action, state=actionState}
end

function Enemy:estimateThreat(state)
  if state.playerNukeLaunched then
    return 10000
  end


  local threat = -5

  if state.power == 1 then
    threat = threat + 5
  end

  if state.fuelPumps == 3 then
    threat = threat + 10
  end

  threat = threat + state.doors * 3

  if state.buttonState == 19 and state.doors == 10 then
    threat = threat + 100
  end

  return threat
end


function newEnemy()
  return setmetatable({
    lastUpdate = 0,
    fear = maxFear / 2,
    lastFear = maxFear / 2,
    lastMessage = 0,
    fearFactor = 1.0,
    assets = {},
  }, Enemy)
end
