require "coup"
require "overlay"
require "teletype"
require "introstate"
require "state"
require "clock"
require "infodrawer"
require "noise"
require "enemy"
require "endstate"

function love.load()
  G = {}
  G.overlays = {}
  G.overlays.teletype = newQuadOverlay("data/overlay/teletype.png", 641, 165, 0, 364, 21)
  G.overlays.teletype_stencil = newQuadOverlay("data/overlay/head_stencil.png", 640, 166, 0, 354, 21)


  G.overlays.paperHolesRight = newQuadOverlay("data/overlay/paper_holes_right.png", 105, 375, 430, 22)
  G.overlays.paperHolesLeft = newQuadOverlay("data/overlay/paper_holes_left.png", 31, 395, 59, 42)
  G.overlays.paperHolesRight.scale_y = 0.997333333

  G.overlays.button = newQuadOverlay("data/overlay/button.png", 310, 274, 829, 366)
  G.pressedButton = love.graphics.newImage("data/overlay/button_pressed.png")

  G.overlays.switch1 = newOverlay("data/overlay/switch1/%04d.png", coup.tables.range(1200, 1201), 1012, 518)
  G.overlays.switch2 = newOverlay("data/overlay/switch2/%04d.png", coup.tables.range(1200, 1201), 1089, 520)
  G.overlays.switch3 = newOverlay("data/overlay/switch3/%04d.png", coup.tables.range(1200, 1201), 1175, 522)
  G.overlays.led1    = newOverlay("data/overlay/led1/%04d.png", coup.tables.range(1202, 1203), 1025, 568)
  G.overlays.led2    = newOverlay("data/overlay/led2/%04d.png", coup.tables.range(1202, 1203), 1107, 570)
  G.overlays.led3    = newOverlay("data/overlay/led3/%04d.png", coup.tables.range(1202, 1203), 1194, 572)

  G.overlays.clock = newQuadOverlay("data/overlay/clock.png", 54, 50, 941, 261)
  --G.introFrame = love.graphics.newImage("data/intro.png")

  G.background = love.graphics.newImage("data/background.png")

  G.paperShader = love.graphics.newShader [[
    uniform Image paperContent;
    uniform float scroll;
    vec4 effect( vec4 color, Image texture, vec2 uv, vec2 screenCoords ) {
      return Texel(paperContent, uv - vec2(0, scroll));
    }
  ]]

  G.maskShader = love.graphics.newShader[[
    vec4 effect(vec4 color, Image texture, vec2 texture_coords, vec2 screen_coords) {
      if (Texel(texture, texture_coords).r > 0.0) {
        discard;
      }
      return vec4(1.0);
    }
  ]]

  local paperMeshData = require("paper_mesh")
  G.paperMesh = love.graphics.newMesh(paperMeshData, "strip", "static")
  G.teletypeFont = love.graphics.newFont("data/uwch.ttf", 13)
  G.infoFont = love.graphics.newFont("data/VarelaRound-Regular.otf", 24)
  G.infoFont2 = love.graphics.newFont("data/VarelaRound-Regular.otf", 48)

  loadNoiseShader()

  G.state = newState()
---[[
  G.state = newIntroState()
 --]]
--[[
  G.state = newEndstate("good")
  --]]
  G.drawCanvas = love.graphics.newCanvas()
  G.shaderTime = 0
  G.lastShaderTime = 0

  G.sounds = {
    nukeLaunch = love.audio.newSource("data/sounds/nukelaunch.ogg", "static"),
    nukeImpact = love.audio.newSource("data/sounds/nukeimpact.ogg", "static"),
    alarm = love.audio.newSource("data/sounds/alarm.ogg", "static"),
    fuelPumps = love.audio.newSource("data/sounds/fuelpumps.ogg", "static"),
    power = love.audio.newSource("data/sounds/power.ogg", "static"),
    doors = love.audio.newSource("data/sounds/doors.ogg", "static"),
    teletype = love.audio.newSource("data/sounds/teletype.ogg", "static"),
    teletype2 = love.audio.newSource("data/sounds/teletype2.ogg", "static"),
    click = love.audio.newSource("data/sounds/click.ogg", "static"),
    cover_open = love.audio.newSource("data/sounds/cover_open.ogg", "static"),
    cover_close = love.audio.newSource("data/sounds/cover_close.ogg", "static"),
    button = love.audio.newSource("data/sounds/button.ogg", "static"),
  }

  G.sounds.alarm:setLooping(true)
  G.sounds.fuelPumps:setLooping(true)
  G.sounds.power:setLooping(true)
  G.sounds.teletype:setLooping(true)
  G.sounds.teletype2:setLooping(true)
end


function love.update(dt)
  G.shaderTime = G.shaderTime + dt
  G.state:update(dt)
end


function love.draw()
  if math.floor(G.shaderTime*20) ~= math.floor(G.lastShaderTime*20) then
    G.noiseShader:send("time", G.shaderTime)
    G.lastShaderTime = G.shaderTime
  end

  love.graphics.setCanvas(G.drawCanvas)
  G.noiseShader:send("col", {1.0, 1.0, 1.0})
  G.noiseShader:send("col_add", {0, 0,0})
  G.state:draw()
  love.graphics.setCanvas()
  love.graphics.setShader(G.noiseShader)
  love.graphics.draw(G.drawCanvas)
  love.graphics.setShader()

end


function love.keypressed(key, scancode, isrepeat)
  
end


function love.keyreleased(key, scancode)

end


function love.mousemoved(x, y, dx, dy, istouch)
end


function love.mousepressed(x, y, button, istouch)

end


function love.mousereleased(x, y, button, istouch)
  G.state:mouseReleased(x, y)
end
