local Overlay = {}
Overlay.__index = Overlay


function Overlay:draw(frame)
  local f = self.frames[(frame or self.curFrame) + 1]
  love.graphics.draw(f[1], f[2], self.x, self.y, 0, self.scale_x or 1, self.scale_y or 1)
end


local function loadFrames(nameTemplate, frames)
  local loadImage = coup.cache(love.graphics.newImage)
  local loadQuad = coup.cache(love.graphics.newQuad)

  local fr = {}
  for i, f in ipairs(frames) do
    local img = loadImage(string.format(nameTemplate, f))
    local w, h = img:getDimensions()
    fr[i] = {img, loadQuad(0,0, w, h, w, h)}
  end

  return fr
end


local function loadQuads(file, qw, qh, n)
  local img = love.graphics.newImage(file)
  local iw, ih = img:getDimensions()

  local cs = math.floor(iw / qw)
  local rs = math.floor(ih / qh)

  local fr = {}
  for r = 1, rs do
    for c = 1, cs do
      if (not n) or (#fr < n) then
        local qx = (c-1) * qw
        local qy = (r-1) * qh
        local q = love.graphics.newQuad(qx, qy, qw, qh, iw, ih)
        fr[#fr+1] = {img, q}
      end
    end
  end

  return fr
end


function newOverlay(nameTemplate, frames, x, y)
  return setmetatable({
    frames = loadFrames(nameTemplate, frames),
    x = x,
    y = y,
    curFrame = 0
  }, Overlay)
end

function newQuadOverlay(file, qw, qh, x, y, n)
  return setmetatable({
    frames = loadQuads(file, qw, qh, n),
    x = x,
    y = y,
    curFrame = 0
  }, Overlay)
end
