local State = {}
State.__index = State


function newState()
  local n = setmetatable({
    teletype = newTeletype(),
    clock = newClock(),
    enemy = newEnemy(),
    switches = {0, 0, 0},
    leds = {0, 0, 0},
    power = false,
    fuelPumps = 0,
    doors = 0,
    ownThreat = 0,
    buttonState = 0,
    buttonDest = 0,
    nukeTime = 0,
    t = 0,
    enemyUpdates = false,
  }, State)

  n.activeAreas = {
    {838, 473, 139, 122, over = function() n:overBigRed() end, click = function() n:openBigRed() end},
    {844, 375, 112, 124, over = function() n:overBigRed2() end, click = function() n:closeBigRed() end},
    {869, 506, 72, 48, over = function() n:overFireMissiles() end, click = function() n:fireMissiles() end},
    {1013, 533, 36, 34, over = function() n:overPower() end, click = function() n:togglePower() end},
    {1091, 533, 37, 35, over = function() n:overFuelPumps() end, click = function() n:toggleFuelPumps() end},
    {1174, 536, 41, 34, over = function() n:overSiloDoors() end, click = function() n:toggleSiloDoors() end},
    {923, 251, 93, 73, over = function() n.info = "Current time: " .. n.clock:printableTime() end, click = function() end},
    {1116, 169, 184, 289, over = function() n.info = "The president is on vacation. You're on your own." end, click = function() end}
  }

  n:startGame()

  return n
end


function State:enemyLaunchesNuke()
  self.enemyNukesLaunched = true
  self.clock:addEvent(3, function()
    self.teletype:print(" ")
    self.teletype:print(" ")
    self.teletype:print("NUCLEAR MISSILE LAUNCH DETECTED")
    self.teletype:print(" ")
    self.teletype:print(" ")
    self.nukeTime = self.t
    G.sounds.alarm:play()
    self.clock:addEvent(8, function()
      self:enemyNukeImpact()
    end, true)
  end, true)
end


function State:enemyNukeImpact()
  G.sounds.alarm:stop()
  G.sounds.power:stop()
  G.sounds.fuelPumps:stop()
  G.sounds.doors:stop()
  G.sounds.nukeImpact:play()
  G.sounds.teletype:stop()
  G.sounds.teletype2:stop()
  self.impact = true
  self.nukeTime = self.t
end


function State:overFireMissiles()
  if self.buttonState == 19 then
    self.info = "Launch MJOLNIR ICBMs"
  end
end


function State:fireMissiles()
  if self.buttonState == 19 then
    self.pressedTime = self.t
    G.sounds.button:play()
    if not self.playerNukeLaunched then
      if self.power == 1 and self.fuelPumps == 3 then
        self.teletype:print("ICBM launch initiated")
        if self.doors == 10 then
          self.clock:addEvent(1, function()
            G.sounds.nukeLaunch:play()
            self.playerNukeLaunched = true
          end, true)
        else
          self.dumbPlayer = true
          self.clock:addEvent(1, function()
            G.sounds.nukeLaunch:play()
            self.playerNukeLaunched = true
            self.clock:addEvent(2, function()
              G.sounds.nukeLaunch:stop()
              self:enemyNukeImpact()
            end, true)
          end, true)
        end
      else
        self.teletype:print("ICBMs not ready:")
        self.teletype:print(" * Main power: " .. ((self.power == 1) and "ON" or "OFF"))
        self.teletype:print(" * Fuel pumps: " .. ((self.fuelPumps == 10) and "ON" or "OFF"))

      end
    end
  end
end


function State:overBigRed()
  if self.buttonState == 0 then
    self.info = "Open Safety Cover"
  end
end

function State:overBigRed2()
  if self.buttonState == 19 then
    self.info = "Close Safety Cover"
  end
end


function State:mouseReleased(x, y)
  for i, o in pairs(self.activeAreas) do
    if x >= o[1] and x <= o[1] + o[3] and y >= o[2] and y <= o[2] + o[4] then
      o.click()
    end
  end
end

function State:overPower()
  local pow = {
    "Enable silo main power",
    "Disable silo main power"
  }

  self.info = pow[self.switches[1]+1]
end

function State:overFuelPumps()
  local pow = {
    "Enable MJOLNIR missile fuel pumps",
    "Disable MJOLNIR missile fuel power"
  }

  self.info = pow[self.switches[2]+1]
end

function State:overSiloDoors()
  local pow = {
    "Open silo doors",
    "Close silo doors"
  }

  self.info = pow[self.switches[3]+1]

end

function State:togglePower()
  self.switches[1] = 1 - self.switches[1]
  self.leds[1] = self.switches[1]
  self.power = self.switches[1]
  local pow = {
    "*** Silo main power shut down",
    "*** Powering up primary circuit breakers"
  }
  self.doorWarning = false
  self.teletype:print(pow[self.power+1])

  G.sounds.click:play()

  if self.power == 1 then
    G.sounds.power:play()

    self:startDoorSound()

  else
    G.sounds.power:stop()
    G.sounds.doors:stop()
  end
end

function State:toggleFuelPumps()
  self.switches[2] = 1 - self.switches[2]

  G.sounds.click:play()
  if self.switches[2] == 1 then
    self.fuelPumps = 0.00001
    self.teletype:print("*** Powering up fuel pumps")
  else
    self.teletype:print("*** Fuel pump shut down")
    self.fuelPumps = 0
  end
end

function State:toggleSiloDoors()
  G.sounds.click:play()
  self.switches[3] = 1 - self.switches[3]
  self.doorWarning = false

  local pow = {
    "*** Closing silo doors",
    "*** Opening silo doors"
  }
  self.doorWarning = false
  self.teletype:print(pow[self.switches[3]+1])
  self:startDoorSound()
end

function State:startDoorSound()
  if self.power == 1 then
    if self.switches[3] == 1 then
      if self.doors < 10 then
        G.sounds.doors:play()
        G.sounds.doors:seek(self.doors)
      end
    else
      if self.doors > 0 then

        G.sounds.doors:play()
        G.sounds.doors:seek(10-self.doors)
      end
    end
  end

end

function State:closeBigRed()
  self.buttonDest = 0
end

function State:openBigRed()
  self.buttonDest = 19
end

function State:checkButton(dt)
  local d = self.buttonDest
  local s = self.buttonState

  if d ~= s then
    if s < d then
      self.buttonState = s + dt*30
      if self.buttonState >= d then
        G.sounds.cover_open:play()
        self.buttonState = d
      end
    else
      self.buttonState = s - dt*30
      if self.buttonState <= d then
        G.sounds.cover_close:play()
        self.buttonState = d
      end
    end
  end
end

function State:startGame()
  local t = self.teletype
  local c = self.clock
  c:addEvent(4, function()
  ---[[
    t:print("****************************************")
    t:print("# 12/14/68 0804h")
    t:print("Morning shift 0800h-1600h")
    t:print("")
    t:print("Department of National Defense")
    t:print("Daily orders for silo MS-1754")
    t:print(" 1......Keep MJOLNIR missiles disarmed")
    t:print(" 2......Observe teletype information")
    t:print(" 3......Don't start a nuclear war!")
    t:print(" 4......React to potential threats")
    t:print("")
    t:print("Major,")
    t:print("a short personal note: The enemy will observe each of our actions.")
    t:print("In an attempt to ensure peace, we will inform the enemy about any steps we take to prepare a launch.")
    t:print("We hope that the balance of terror will keep them from doing something stupid.")
    t:print("Make sure they understand that we are prepared, should they attempt to strike first.")
    t:print("But don't make them think that we are launching a preventive strike.")
    t:print("Please pay close attention to your TELEMATICS teletypewriter.")
  --]]
    t:print("     - Admiral Jones")
    t:print("", function()
      self.enemyUpdates = true
    end)
  end, true)


  c:addEvent(16*60, function()
    self:endGame()
  end)
end


function State:checkEvent()
  local c = self.clock
  --print("Event")
  c:addEvent(5, function() self:checkEvent() end, true)
end


function State:endGame()
  print("game ended")
end

function State:checkPumpStatus(dt)
  if self.switches[2] == 1 then
    if self.power == 1 then
      if self.fuelPumps < 3 then
        self.fuelPumps = self.fuelPumps+dt
        if self.fuelPumps >= 3 then
          self.fuelPumps = 3
          self.teletype:print("*** Fuel pumps at full capacity")
          self.leds[2] = 1
        else
          self.leds[2] = math.floor(self.fuelPumps * 4) % 2
        end
      end
    else
      if self.fuelPumps > 0 then
        self.fuelPumps = 0
        self.teletype:print("*** FUEL PUMP POWER FAILURE")
        self.leds[2] = 0
      end
    end
  else
    self.leds[2] = 0
  end

  if self.fuelPumps > 0 and not self.playerNukeLaunched then
    G.sounds.fuelPumps:setPitch(self.fuelPumps / 3)
    G.sounds.fuelPumps:play()
  else
    G.sounds.fuelPumps:stop()
  end
end

function State:checkSiloDoors(dt)
  if self.power == 1 then
    if self.switches[3] == 1 then
      if self.doors < 10 then
        self.doors = self.doors + dt
        if self.doors >= 10 then
          self.doors = 10
          self.teletype:print("*** Silo doors open")
          self.leds[3] = 1
        end
      end
    else
      if self.doors > 0 then
        self.doors = self.doors - dt
        if self.doors <= 0 then
          self.doors = 0
          self.teletype:print("*** Silo doors closed")
          self.leds[3] = 0
        end

      end
    end
  else
    if self.switches[3] == 1  and self.doors ~= 10 then
      if not self.doorWarning then
        self.doorWarning = true
        self.teletype:print("*** SILO DOOR POWER FAILURE")
        self.teletype:print("*** WARNING, DOORS STUCK AT " .. math.floor(self.doors * 10) .. "%")
      end
    end

    if self.switches[3] == 0  and self.doors ~= 0 then
      if not self.doorWarning then
        self.doorWarning = true
        self.teletype:print("*** SILO DOOR POWER FAILURE")
        self.teletype:print("*** WARNING, DOORS STUCK AT " .. math.floor(self.doors * 10) .. "%")
      end
    end
  end

  if (self.doors ~= 0  and self.switches[3] == 0) or (self.doors ~= 10 and self.switches[3] == 1) then
    self.leds[3] = math.floor(self.t * 4) % 2
  end
end


function State:update(dt)
  self.t = self.t + dt
  if not self.impact and not self.won then
    self:mouseMoved(love.mouse.getPosition())
    self.teletype:update(dt)
    self.clock:update(dt)
    self:checkPumpStatus(dt)
    self:checkSiloDoors(dt)
    self:checkButton(dt)

    if self.clock.time >= 16 then
      self:shiftOver()
    end

    if self.enemyUpdates and not self.enemyNukesLaunched then
      self.enemy:estimateState(self)
    end
  end

  if self.impact then
    if self.t - self.nukeTime > 3 then
      if self.dumbPlayer then
        G.state = newEndstate("dumb")
      elseif self.playerNukeLaunched then
        G.state = newEndstate("mutual")
      else
        G.state = newEndstate("onlyPlayer")
      end
    end
  end

  if self.won then
    if self.t - self.nukeTime > 3 then
      G.state = newEndstate("good")
    end
  end
end

function State:shiftOver()
  if (not self.enemyNukesLaunched) and (not self.playerNukeLaunched) then
    self.won = true
    self.nukeTime = self.t
  end
  
end


function State:toggleSwitch(idx)
  self.switches[idx] = 1-self.switches[idx]
end


function State:mouseMoved(x, y)
  self.info = nil
  for i, o in pairs(self.activeAreas) do
    if x >= o[1] and x <= o[1] + o[3] and y >= o[2] and y <= o[2] + o[4] then
      o.over()
    end
  end
end


function State:draw()
  if not self.impact and self.nukeTime > 0 then
    local fac = math.cos((self.t - self.nukeTime) * 10) * 0.4 + 0.6
    G.noiseShader:send("col", {1.0, fac, fac})
  end

  if self.impact then
    local fac = math.min(self.t - self.nukeTime, 1.0)
    G.noiseShader:send("col_add", {fac, fac, fac})
  end

  if self.won then
    local fac = math.min(1-(self.t - self.nukeTime)/3, 1.0)
    G.noiseShader:send("col", {fac, fac, fac})
  end

  love.graphics.draw(G.background)
  G.overlays.button:draw(math.floor(self.buttonState))

  if self.pressedTime and self.pressedTime > self.t - 0.3 then
    love.graphics.draw(G.pressedButton, 829, 366)
  end

  for i = 1, 3 do
    local swO = G.overlays["switch" .. i]
    swO:draw(self.switches[i])
    local ledO = G.overlays["led" .. i]
    ledO:draw(self.leds[i])
  end
  self.teletype:draw()
  if self.info then
    drawInfo(self.info)
  end

  self.clock:draw()
end
